@@ -209,6 +209,65 @@ class FilesAPI(GenericAPI):
 
         return output
 
+    def get_all_from_location(self, space_id, path=None, file_type=None):
+        """
+        Get all files/folders in a given file system location.
+
+        Parameters
+        ----------
+        space_id : str
+            The id of the workspace where the files of interest are stored.
+        path : str
+            The parent folder path of interest.  This will be a comma-separated
+            list of ids up to and including the parent folder id.
+            Ex:  ",uuid,uuid,uuid,"
+        file_type : hyperthought.api.files.FilesAPI.FileType or None
+            An enum instance specifying the type of files to get, i.e. files,
+            folders, or both.  Will get both if None.
+
+        Returns
+        -------
+        A list of dicts containing information on files and/or folders in a
+        given file system location.
+        """
+        # Validate inputs.
+
+        if not isinstance(space_id, str):
+            raise ValueError(
+                "The space_id parameter to get_all_from_location must be a "
+                "string."
+            )
+
+        if path is not None and not isinstance(path, str):
+            raise ValueError(
+                "The path parameter to get_all_from_location must be a string "
+                "or None."
+            )
+
+        # Set pagination parameters.
+        start = 0
+        length = 25
+
+        # Create list to collect output.
+        files = []
+
+        while True:
+            new_files = self.get_from_location(
+                space_id=space_id,
+                path=path,
+                file_type=file_type,
+                start=start,
+                length=length,
+            )
+
+            if not new_files:
+                break
+
+            files.extend(new_files)
+            start += length
+
+        return files
+
     def get_id(self, name, space_id, path=None):
         """
         Get an id for a file/folder with a given name at a given location.
