@@ -38,7 +38,7 @@ class UnionFind:
             get_id = lambda x: x
 
         if not callable(get_id):
-            return ValueError("get_id must be a callable")
+            raise ValueError("get_id must be a callable")
 
         # TODO:  Validate that get_id takes one parameter and returns a
         #        hashable value.
