@@ -14,6 +14,8 @@ from xml.etree.ElementTree import (
 BUFFER = 100
 MAX_JITTER = BUFFER // 2
 JITTER_STEP = 10
+DECISION_WIDTH = 80
+DECISION_HEIGHT = 80
 PROCESS_WIDTH = 80
 PROCESS_HEIGHT = 80
 WORKFLOW_WIDTH = 120
@@ -21,8 +23,8 @@ WORKFLOW_HEIGHT = 60
 
 # Width delta between adjacent elements in a row or column.
 # (Amount to add to get from one element to the next.)
-ADD_WIDTH = max(PROCESS_WIDTH, WORKFLOW_WIDTH) + BUFFER
-ADD_HEIGHT = max(PROCESS_HEIGHT, WORKFLOW_HEIGHT) + BUFFER
+ADD_WIDTH = max(DECISION_WIDTH, PROCESS_WIDTH, WORKFLOW_WIDTH) + BUFFER
+ADD_HEIGHT = max(DECISION_HEIGHT, PROCESS_HEIGHT, WORKFLOW_HEIGHT) + BUFFER
 
 NAME_KEY = 'name'
 TYPE_KEY = 'type'
@@ -30,13 +32,21 @@ COLUMN_KEY = 'column'
 ROW_KEY = 'row'
 ID_KEY = 'id'
 CLIENT_ID_KEY = 'client_id'
+# Values for all successors keys should be sets, for efficiency, though lists
+# will still work.
 SUCCESSORS_KEY = 'successors'
+# These keys are only relevant to decisions.
+# Decisions must still have a SUCCESSORS_KEY in addition to the following.
+YES_SUCCESSORS_KEY = 'yes_successors'
+NO_SUCCESSORS_KEY = 'no_successors'
 
+DECISION_TYPE = 'decision'
 PROCESS_TYPE = 'process'
 WORKFLOW_TYPE = 'workflow'
 
 PROCESS_STYLE = "rounded=0;whiteSpace=wrap;html=1;"
 WORKFLOW_STYLE = "shape=process;whiteSpace=wrap;html=1;"
+DECISION_STYLE = "rhombus;whiteSpace=wrap;html=1;"
 EDGE_STYLE = (
     "edgeStyle=orthogonalEdgeStyle;"
     "rounded=0;"
@@ -48,12 +58,38 @@ EDGE_STYLE = (
     "jettySize=auto;"
     "orthogonalLoop=1;"
 )
+YES_EDGE_STYLE = (
+    "edgeStyle=orthogonalEdgeStyle;"
+    "rounded=0;"
+    "html=1;"
+    "exitX=0.5;"
+    "exitY=0;"
+    "entryX=0;"
+    "entryY=0.5;"
+    "entryDx=0;"
+    "entryDy=0;"
+    "jettySize=auto;"
+    "orthogonalLoop=1;"
+)
+NO_EDGE_STYLE = (
+    "edgeStyle=orthogonalEdgeStyle;"
+    "rounded=0;"
+    "html=1;"
+    "exitX=0.5;"
+    "exitY=1;"
+    "entryX=0;"
+    "entryY=0.5;"
+    "entryDx=0;"
+    "entryDy=0;"
+    "jettySize=auto;"
+    "orthogonalLoop=1;"
+)
 
 
 # This will only be used for edges and empty cells.
 # Nodes need to correspond to database objects, so the client ids will be
 # precomputed.
-def get_client_id():
+def _get_client_id():
     return str(uuid.uuid4())
 
 
@@ -122,23 +158,43 @@ def _get_y(logical_row):
 
     This will be the position of the top left corner.
     """
-    return BUFFER + ADD_HEIGHT * logical_row
+    # Add additional buffer to accommodate edges that go above the row.
+    # (This will happen for decision 'Yes' paths.)
+    return BUFFER * 2 + ADD_HEIGHT * logical_row
 
 
-def _get_port_y(node):
+def _get_out_port_x(node):
     """
-    Get the absolute y position of a port for a node.
+    Get the absolute x position for an out port for a source node.
+    """
+    source_node_type = node[TYPE_KEY]
+
+    if source_node_type == DECISION_TYPE:
+        width_adjustment = DECISION_WIDTH // 2
+    elif source_node_type == PROCESS_STYLE:
+        width_adjustment = PROCESS_WIDTH
+    else:
+        width_adjustment = WORKFLOW_WIDTH
+
+    return _get_x(logical_column=node[COLUMN_KEY]) + width_adjustment
+
+
+def _get_in_port_y(node):
+    """
+    Get the absolute y position of an in port for a node.
 
     Ports (edge connection points) will be halfway from top to bottom.
     """
-    if node[TYPE_KEY] == PROCESS_TYPE:
-        half_height = PROCESS_HEIGHT // 2
+    if node[TYPE_KEY] == DECISION_TYPE:
+        height_adjustment = DECISION_HEIGHT // 2
+    elif node[TYPE_KEY] == PROCESS_TYPE:
+        height_adjustment = PROCESS_HEIGHT // 2
     elif node[TYPE_KEY] == WORKFLOW_TYPE:
-        half_height = WORKFLOW_HEIGHT // 2
+        height_adjustment = WORKFLOW_HEIGHT // 2
     else:
         raise Exception(f"Invalid type detected: {node[TYPE_KEY]}")
 
-    return _get_y(logical_row=node[ROW_KEY]) + half_height
+    return _get_y(logical_row=node[ROW_KEY]) + height_adjustment
 
 
 def _get_first_edge_x(source_node):
@@ -152,16 +208,13 @@ def _get_first_edge_x(source_node):
     return _get_x(logical_column=source_node[COLUMN_KEY] + 1) - BUFFER // 2
 
 
-def _get_between_edge_y(source_node):
+def _get_between_edge_y(logical_row):
     """
-    Get the absolute y position of the second point in an edge connecting
-    nodes in non-adjacent columns.
+    Get the absolute y position of the horizontal edge connecting nodes.
 
-    Edges will always drop to go around in-between columns.
-
-    This will be a half-buffer above the next row down.
+    This y position will be adjusted by jitter elsewhere.
     """
-    return _get_y(logical_row=source_node[ROW_KEY] + 1) - BUFFER // 2
+    return _get_y(logical_row=logical_row + 1) - BUFFER // 2
 
 
 def _get_last_edge_x(target_node):
@@ -191,7 +244,11 @@ def _create_node(node, parent_id):
     -------
     An XML element representing the process.
     """
-    if node[TYPE_KEY] == PROCESS_TYPE:
+    if node[TYPE_KEY] == DECISION_TYPE:
+        style = DECISION_STYLE
+        width = DECISION_WIDTH
+        height = DECISION_HEIGHT
+    elif node[TYPE_KEY] == PROCESS_TYPE:
         style = PROCESS_STYLE
         width = PROCESS_WIDTH
         height = PROCESS_HEIGHT
@@ -221,10 +278,93 @@ def _create_node(node, parent_id):
     return cell_element
 
 
+def _create_decision_edge(source_node, target_node, parent_id, jitter_manager):
+    """Create an edge from a decision node to a successor/target node."""
+    is_yes_edge = target_node[ID_KEY] in source_node[YES_SUCCESSORS_KEY]
+
+    cell_element = Element('mxCell', {
+        'id': _get_client_id(),
+        'style': YES_EDGE_STYLE if is_yes_edge else NO_EDGE_STYLE,
+        'value': 'Yes' if is_yes_edge else 'No',
+        'edge': '1',
+        'parent': parent_id,
+        'source': source_node[CLIENT_ID_KEY],
+        'target': target_node[CLIENT_ID_KEY],
+    })
+
+    geometry_element = SubElement(cell_element, 'mxGeometry', {
+        'relative': '1',
+        'as': 'geometry',
+    })
+
+    # Add an array of points as needed.
+    # This will always be done, even if the nodes are in the same row and
+    # adjacent columns.  The reason is that the line across will not be
+    # straight if the nodes have different types.
+
+    # 'Yes' paths will go above, all others will go below.
+    if (
+        source_node[TYPE_KEY] == DECISION_TYPE
+        and
+        target_node[ID_KEY] in source_node[YES_SUCCESSORS_KEY]
+    ):
+        source_logical_row = source_node[ROW_KEY] - 1
+    else:
+        source_logical_row = source_node[ROW_KEY]
+
+    source_logical_column = source_node[COLUMN_KEY]
+    target_logical_column = target_node[COLUMN_KEY]
+    assert source_logical_column < target_logical_column, (
+        "The source node for an edge should be to the left of the "
+        "target node.")
+
+    # Add the array element.
+    array_element = SubElement(geometry_element, 'Array', {'as': 'points'})
+
+    # Move up or down from yes or no port, respectively.
+    jitter = jitter_manager.next(
+        is_row=True,
+        row_or_column_number=source_logical_row,
+    )
+    first_y = _get_between_edge_y(source_logical_row) + jitter
+    SubElement(array_element, 'mxPoint', {
+        'x': str(_get_out_port_x(source_node)),
+        'y': str(first_y),
+    })
+
+    # Move across to the next x-position.
+    jitter = jitter_manager.next(
+        is_row=False,
+        row_or_column_number=target_logical_column - 1,
+    )
+    last_x = _get_last_edge_x(target_node) + jitter
+    SubElement(array_element, 'mxPoint', {
+        'x': str(last_x),
+        'y': str(first_y),
+    })
+
+    # Move down to y-position of in port for successor.
+    last_y = _get_in_port_y(target_node)
+    SubElement(array_element, 'mxPoint', {
+        'x': str(last_x),
+        'y': str(last_y),
+    })
+
+    return cell_element
+
+
 def _create_edge(source_node, target_node, parent_id, jitter_manager):
     """Create an edge between two nodes in the workflow."""
+    if source_node[TYPE_KEY] == DECISION_TYPE:
+        return _create_decision_edge(
+            source_node=source_node,
+            target_node=target_node,
+            parent_id=parent_id,
+            jitter_manager=jitter_manager,
+        )
+
     cell_element = Element('mxCell', {
-        'id': get_client_id(),
+        'id': _get_client_id(),
         'style': EDGE_STYLE,
         'edge': '1',
         'parent': parent_id,
@@ -238,7 +378,7 @@ def _create_edge(source_node, target_node, parent_id, jitter_manager):
     })
 
     # Add an array of points as needed.
-    # This will always be done, event if the nodes are in the same row and
+    # This will always be done, even if the nodes are in the same row and
     # adjacent columns.  The reason is that the line across will not be
     # straight if the nodes have different types.
 
@@ -257,12 +397,13 @@ def _create_edge(source_node, target_node, parent_id, jitter_manager):
         is_row=False,
         row_or_column_number=source_logical_column,
     )
+
     first_x = _get_first_edge_x(source_node) + jitter
     SubElement(array_element, 'mxPoint', {
         'x': str(first_x),
-        'y': str(_get_port_y(source_node)),
+        'y': str(_get_in_port_y(source_node)),
     })
-    last_y = _get_port_y(target_node)
+    last_y = _get_in_port_y(target_node)
 
     if source_logical_column == target_logical_column - 1:
         SubElement(array_element, 'mxPoint', {
@@ -274,7 +415,7 @@ def _create_edge(source_node, target_node, parent_id, jitter_manager):
             is_row=True,
             row_or_column_number=source_logical_row,
         )
-        second_y = _get_between_edge_y(source_node) + jitter
+        second_y = _get_between_edge_y(source_logical_row) + jitter
         SubElement(array_element, 'mxPoint', {
             'x': str(first_x),
             'y': str(second_y),
@@ -331,8 +472,8 @@ def create_xml(logical_children):
     ----------
     logical_children : list of dict
         Dictionaries for processes/workflows to put on the canvas.
-        Expected keys include 'id', 'name', 'type', 'successors', 'row', and
-        'column'.
+        Expected keys include 'id', 'client_id', 'name', 'type', 'successors',
+        'row', and 'column'.
 
     Return
     ------
@@ -376,10 +517,10 @@ def create_xml(logical_children):
 
     root_element = SubElement(graph_element, 'root')
 
-    empty_id = get_client_id()
+    empty_id = _get_client_id()
     SubElement(root_element, 'mxCell', {'id': empty_id})
 
-    parent_id = get_client_id()
+    parent_id = _get_client_id()
     SubElement(root_element, 'mxCell', {
         'id': parent_id,
         'parent': empty_id,
