@@ -1,15 +1,27 @@
 """
-This is a convenience module designed to manage OpenID Connect authorization
-to the HyperThought system.  Detailed usage instructions can be found in the
-Authorization class.
+Authentication classes used to interact with the HyperThought API.
+
+Each app in HyperThought will have an associated API class in this package.
+For example, the files app has hyperthought.api.files.FilesAPI.
+These API classes are initialized with a self-refreshing auth object,
+as defined below.
+
+Two auth objects are available:  TokenAuthentication and ClientAuthentication.
+The first receives encoded information from the profile page in HyperThought,
+uses a bearer token to authenticate API calls, and automatically updates the
+bearer token internally using a refresh token.  The second uses a client id
+and secret.
 """
 
+import abc
 import base64
 from datetime import datetime
 from datetime import timedelta
 import json
+import sys
 from threading import Thread
 from time import sleep
+from warnings import warn
 
 import dateutil.parser
 from dateutil.tz import tzlocal
@@ -23,7 +35,6 @@ REFRESH_WINDOW = 300
 # Sleep time between refresh attempts.
 REFRESH_RETRY_SLEEP_TIME = 1
 
-
 # Disable insecure request warnings.
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
@@ -32,9 +43,34 @@ class AuthExpirationException(Exception):
     pass
 
 
-class Authorization:
+class Authentication(abc.ABC):
     """
-    Authorization manager for access to HyperThought via the REST API.
+    Abstract base class for authentication classes.  Defines common interface.
+    """
+
+    def __init__(self, verify=True):
+        self._verify = verify
+
+    @property
+    def verify(self):
+        return self._verify
+
+    @abc.abstractmethod
+    def get_headers():
+        pass
+
+    @abc.abstractmethod
+    def get_cookies():
+        pass
+
+    @abc.abstractmethod
+    def get_base_url():
+        pass
+
+
+class TokenAuthentication(Authentication):
+    """
+    Authentication manager for token-based access to HyperThought API.
 
     Parameters
     ----------
@@ -64,7 +100,7 @@ class Authorization:
     -----
     1.  Go to the My Account page in HyperThought.  Copy the API Access code
         to the clipboard.
-    2.  Instantiate an Authorization object using this code as the parameter
+    2.  Instantiate an Authentication object using this code as the parameter
         value (auth_payload argument).
     3.  Make calls to the API using headers and cookies obtained from the
         get_headers and get_cookies methods, as well as the verify property.
@@ -86,7 +122,7 @@ class Authorization:
     )
 
     def __init__(self, auth_payload, verify=True, delayed_refresh=False):
-        self._verify = bool(verify)
+        super().__init__(verify)
         self._delayed_refresh = bool(delayed_refresh)
         auth_payload = json.loads(base64.b64decode(auth_payload))
 
@@ -125,14 +161,6 @@ class Authorization:
         self._refresh_thread.setDaemon(True)
         self._refresh_thread.start()
 
-    @property
-    def verify(self):
-        return self._verify
-
-    @verify.setter
-    def verify(self, value):
-        self._verify = value
-
     def get_base_url(self):
         """Get the base url for the auth client."""
         return self._base_url
@@ -266,3 +294,126 @@ class Authorization:
                 f"Status code: {r.status_code}. "
                 f"Response: {r.content.decode()}."
             )
+
+
+class Authorization(TokenAuthentication):
+    """
+    Deprecated Authentication manager for token-based access to HyperThought
+    API, maintained for backwards compatibility.  Deprecated as of version
+    0.9.26.4, replaced by TokenAuthentication.
+    """
+    def __init_subclass__(cls, **kwargs):
+        """This throws a deprecation warning on subclassing."""
+        warn(f'{cls.__name__} is deprecated. Use TokenAuthentication instead',
+             DeprecationWarning, stacklevel=2)
+        super().__init_subclass__(**kwargs)
+
+    def __init__(self, auth_payload, verify=True, delayed_refresh=False):
+        """This throws a deprecation warning on initialization."""
+        warn(f'{self.__class__.__name__} is deprecated. '
+             'Use TokenAuthentication instead.',
+             DeprecationWarning, stacklevel=2)
+        super().__init__(auth_payload, verify, delayed_refresh)
+
+
+class ClientAuthentication(Authentication):
+    """
+    Authentication manager for OIDC client-based access to HyperThought API.
+
+    Parameters
+    ----------
+    client_id : str
+        The service account client id.
+    client_secret : str
+        The service account client secret.
+    auth_base_url : str
+        The base url of the HyperThought workspace that the service account
+        is linked to.
+    verify : bool
+        Determine whether SSL verification will be used.
+    """
+
+    def __init__(self, client_id, client_secret, auth_base_url, verify=True):
+        super().__init__(verify)
+        self.AUTH_POST_DATA = "grant_type=client_credentials"
+        self.AUTH_CONTENT_TYPE = "application/x-www-form-urlencoded"
+
+        self.client_id = client_id
+        self.client_secret = client_secret
+        self.auth_base_url = auth_base_url
+        self.auth_target_url = f"{auth_base_url}/openid/token/"
+
+        # This cookie is needed to avoid being redirected to the HyperThought
+        # banner page.
+        self._cookies = {
+            'dodAccessBanner': 'true'
+        }
+
+        # This data members will be set by self._update_auth,
+        # which will be called as needed by self.get_headers.
+        self._access_token = None
+        self._expiration = None
+
+    def get_headers(self):
+        """Get headers for authenticated REST API requests."""
+
+        if self._is_expired():
+            self._update_auth()
+
+        return {
+            'Authorization': f'Bearer {self._access_token}'
+        }
+
+    def get_cookies(self):
+        """Get cookies for authenticated REST API requests."""
+        return dict(self._cookies)
+
+    def get_base_url(self):
+        """Get the base url for the auth client."""
+        return self.auth_base_url
+
+    def _update_auth(self):
+        """
+        Update bearer token used for authentication.
+
+        The bootstrap request will yield
+        """
+        r = requests.post(
+            url=self.auth_target_url,
+            data=self.AUTH_POST_DATA,
+            headers={
+                'Content-Type': self.AUTH_CONTENT_TYPE,
+            },
+            auth=requests.auth.HTTPBasicAuth(
+                self.client_id,
+                self.client_secret,
+            ),
+            cookies=self.get_cookies(),
+            verify=self.verify,
+        )
+
+        # No need to raise an exception.
+        # The expired token will result in an API error when used.
+        # Retrying the request will result in another invocation of
+        # get_headers, thus another invocation of this method.
+        if r.status_code >= 300:
+            print(
+                f"Authentication failure. Status code: {r.status_code}. "
+                f"Response content: {r.content}",
+                file=sys.stderr,
+                flush=True,
+            )
+            return
+
+        self._access_token = r.json()['access_token']
+        expires_in = r.json()['expires_in']
+        now = datetime.now(tzlocal())
+        self._expiration = now + timedelta(seconds=int(expires_in))
+
+    def _is_expired(self):
+        """Determine whether the current auth token is expired."""
+        return (
+            self._expiration is None
+            or
+            self._expiration <= datetime.now(tzlocal())
+        )
