@@ -2,9 +2,26 @@
 Miscellaneous  functionality relating to metadata.
 """
 
+import collections
 import datetime
 
 
+def from_api_format(metadata):
+    """Convert metadata list from API format to list of MetadataItems."""
+    return [
+        MetadataItem.from_api_format(item)
+        for item in metadata
+    ]
+
+
+def to_api_format(metadata):
+    """Convert list of MetadataItems to API formatted metadata."""
+    return [
+        item.to_api_format()
+        for item in metadata
+    ]
+
+
 class MetadataItem:
     """
     Convenience class for working with a metadata element.
@@ -19,7 +36,7 @@ class MetadataItem:
         The units associated with the metadata item.
     annotation : str or None
         An annotation (comment) associated with the metadata item.
-    type : str or None
+    type_ : str or None
         The type of value.  Must be one of self.VALID_TYPES.
     """
     # Non-link types will be based on XML types.
@@ -229,3 +246,55 @@ class MetadataItem:
             output['annotation'] = self.annotation
 
         return output
+
+    @classmethod
+    def from_api_format(cls, api_metadata_item):
+        """
+        Translate item data from API format to object of class.
+
+        Parameters
+        ----------
+        api_metadata_item : dict
+            Metadata item as returned by HyperThought REST API.
+
+        Returns
+        -------
+        MetadataItem object representing the same data.
+        """
+        # Validate api_metadata_item.
+
+        if not isinstance(api_metadata_item, collections.abc.Mapping):
+            raise ValueError(
+                "api_metadata_item must be a dict or other mapping")
+
+        if "keyName" not in api_metadata_item:
+            raise ValueError("api_metadata_item must have a 'keyName' key")
+
+        if "value" not in api_metadata_item:
+            raise ValueError("api_metadata_item must have a 'value' key")
+
+        if "link" not in api_metadata_item["value"]:
+            raise ValueError(
+                "api_metadata_item['value'] must have a 'link' key")
+
+        if "type" not in api_metadata_item["value"]:
+            raise ValueError(
+                "api_metadata_item['value'] must have a 'type' key")
+
+        kwargs = {
+            "key": api_metadata_item["keyName"],
+            "value": api_metadata_item["value"]["link"],
+        }
+
+        # Allow the constructor to infer the type.
+        # Exception:  object links (type == 'link').
+        if api_metadata_item["value"]["type"] == "link":
+            kwargs["type_"] = "link"
+
+        if "annotation" in api_metadata_item:
+            kwargs["annotation"] = api_metadata_item["annotation"]
+
+        if "unit" in api_metadata_item:
+            kwargs["units"] = api_metadata_item["unit"]
+
+        return cls(**kwargs)
